import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:insurance_card_scanner/models/scanned_card.dart';
import 'package:insurance_card_scanner/models/card_mapper.dart';
import 'package:insurance_card_scanner/models/card_response_model.dart';
import 'package:webview_flutter/webview_flutter.dart';

const captureConsole = '''
    var oldLog = console.log;
    console.log = function (message) {
      const _message = {
        command: 'console.log',
        message: message
      }
      window.FlutterWebView.postMessage(JSON.stringify(_message));
      oldLog.apply(console, arguments);
    };

    var oldDebug = console.debug;
    console.debug = function (message) {
      const _message = {
        command: 'console.debug',
        message: message
      }
      window.FlutterWebView.postMessage(JSON.stringify(_message));
      oldDebug.apply(console, arguments);
    };

    var oldError = console.error;
    console.error = function (message) {
      const _message = {
        command: 'console.error',
        message: message
      }
      window.FlutterWebView.postMessage(JSON.stringify(_message));
      oldError.apply(console, arguments);
    };
''';

typedef ScanErrorCallback = Function(String?);
typedef ScanSuccessCallback = Function(ScannedCard);

class CardScanner extends StatefulWidget {
  final String sessionToken;
  final ScanSuccessCallback onCardScanSuccess;
  final VoidCallback? onCardScanCancel;
  final ScanErrorCallback? onCardScanError;
  final VoidCallback? onWebViewLoadError;
  final VoidCallback? onWebViewCreated;
  final bool live;
  final bool debug;

  const CardScanner({
    Key? key,
    required this.sessionToken,
    required this.onCardScanSuccess,
    this.onCardScanCancel,
    this.onCardScanError,
    this.onWebViewLoadError,
    this.onWebViewCreated,
    this.live = false,
    this.debug = false,
  }) : super(key: key);

  @override
  State<CardScanner> createState() => _CardScannerState();
}

class _CardScannerState extends State<CardScanner> {
  final GlobalKey webViewKey = GlobalKey();

  String get webViewUrl => widget.live
      ? 'https://api.cardscan.ai/v1/webview'
      : 'https://sandbox.cardscan.ai/v1/webview';

  late WebViewController _myController;

  void processMessage(JavascriptMessage payload) {
    try {
      final message = jsonDecode(payload.message);
      switch (message['command']) {
        case 'cancel':
          _log('cardScanCancel');
          widget.onCardScanCancel?.call();
          break;
        case 'success':
          _log('cardScanSuccess');
          final cardJson = message['card'];
          final card = cardJson is Map
              ? CardResponseModel.fromJson(cardJson).toCard()
              : null;
          if (card != null) {
            widget.onCardScanSuccess.call(card);
          } else {
            widget.onCardScanError?.call(null);
          }
          break;
        case 'error':
          final error = message['error'];
          final errorMessage = error is Map ? error['message'] : null;
          _log("cardScanError - $error");
          widget.onCardScanError?.call(errorMessage);
          break;
        case 'syserror':
        case 'console.log':
        case 'console.error':
        case 'console.debug':
          var error = message['message'];
          _log("WebView - $error");
          break;
        default:
          _log("Unknown command received from JS - $message");
      }
    } catch (e) {
      _log("Failed to parse JavascriptMessage - ${payload.message} $e");
    }
  }

  @override
  Widget build(BuildContext context) {
    final osName = Platform.operatingSystem.toLowerCase();
    var version = Uri.encodeComponent(Platform.operatingSystemVersion);
    var package = '0.1.0';

    return WebView(
        key: webViewKey,
        initialUrl:
            "$webViewUrl?platform=flutter&os=$osName&version=$version&package=$package",
        // initialUrl: "http://localhost:3000/?platform=flutter",
        onWebViewCreated: (WebViewController webViewController) {
          _myController = webViewController;
        },
        onPageFinished: (url) {
          _log("onPageFinished: $url");

          final config = {
            "live": widget.live,
            "debug": widget.debug,
            "sessionToken": widget.sessionToken,
            //content: content,
            "enableCameraPermissionModal": false,
          };

          final javaScript = '''
            if (window.loadCardScanView != undefined) {
              window.loadCardScanView('${jsonEncode(config)}');
            } else {
              window.addEventListener('CardScanViewLoaded', function (e) {
                window.loadCardScanView('${jsonEncode(config)}');
              });
            }

            $captureConsole

            //setup error monitoring
            window.addEventListener('error', function (e) {
              const message = {
                command: 'syserror',
                message: e.message
              }
              window.ReactNativeWebView.postMessage(JSON.stringify(message));
            });

            console.debug("injectedJavaScript -- complete");

          ''';

          _myController
              .runJavascriptReturningResult(javaScript)
              .then((ret) => {_log("cardScanInit: $ret")});
        },
        onWebResourceError: (WebResourceError error) {
          _log("onWebResourceError: $error");
          widget.onWebViewLoadError?.call();
        },
        debuggingEnabled: widget.debug,
        javascriptMode: JavascriptMode.unrestricted,
        javascriptChannels: {
          JavascriptChannel(
            name: 'FlutterWebView',
            onMessageReceived: processMessage,
          )
        },
        zoomEnabled: false,
        userAgent: "insurance_card_scanner - $package",
        initialMediaPlaybackPolicy: AutoMediaPlaybackPolicy.always_allow,
        allowsInlineMediaPlayback: true);
  }

  void _log(String message) {
    if (kDebugMode) {
      debugPrint('CardScanModal $message');
    }
  }
}
